@@ -14,8 +14,7 @@ pub fn export_csv(csv_file: String, db_file: String) -> Result<()> {
     let (start, end) = full_timerange();
     debug!("start:{}, end:{}", start, end);
 
-    let db = Database::open(db_file).context("open 1history db")?;
-
+    let db = Database::open(db_file).context("open 1History DB")?;
     let f = OpenOptions::new()
         .create(true)
         .write(true)
@@ -25,7 +24,7 @@ pub fn export_csv(csv_file: String, db_file: String) -> Result<()> {
     let mut buf_writer = BufWriter::new(f);
 
     buf_writer.write_all(b"time,title,url,visit_type\n")?;
-    let visits = db.select_visits(start, end)?;
+    let visits = db.select_visits(start, end, None)?;
     let len = visits.len();
     for visit in visits {
         buf_writer.write_all(
