@@ -1,9 +1,9 @@
 use crate::{
     database::Database,
-    types::{ErrorMessage, ServerError, TimeRange},
+    types::{ClientError, DetailsQueryParams, ErrorMessage, IndexQueryParams, ServerError},
     util::{
         minijinja_format_as_hms, minijinja_format_as_ymd, minijinja_format_title,
-        now_as_unixepoch_ms,
+        tomorrow_midnight, ymd_midnight,
     },
 };
 use anyhow::{Context, Error, Result};
@@ -58,10 +58,17 @@ impl Server {
         warp::any().map(move || db.clone())
     }
 
-    async fn details(db: Arc<Database>, day: i64) -> Result<impl Reply, Rejection> {
-        let start = day;
+    async fn details(
+        db: Arc<Database>,
+        ymd: String,
+        query_params: DetailsQueryParams,
+    ) -> Result<impl Reply, Rejection> {
+        let start = ymd_midnight(&ymd).map_err(ClientError::from)?;
         let end = start + 3_600_000 * 24;
-        let visit_details = db.select_visits(start, end).map_err(ServerError::from)?;
+        let keyword = query_params.keyword;
+        let visit_details = db
+            .select_visits(start, end, keyword.clone())
+            .map_err(ServerError::from)?;
 
         let asset = Asset::get("details.html").unwrap();
         let index_tmpl: &str =
@@ -76,25 +83,36 @@ impl Server {
         let tmpl = env.get_template("details").unwrap();
         let body = tmpl
             .render(context!(
-                day => day,
+                ymd => ymd,
+                ymd_ts => start,
                 visit_details => visit_details,
+                version => clap::crate_version!(),
+                keyword => keyword.unwrap_or_else(|| "".to_string()),
             ))
             .map_err(|e| ServerError::from(Error::from(e)))?;
 
         Ok(reply::html(body))
     }
 
-    async fn index(db: Arc<Database>, time_range: TimeRange) -> Result<impl Reply, Rejection> {
-        let end = time_range.end.unwrap_or_else(|| {
-            let now = now_as_unixepoch_ms();
-            now - (now % (3_600_000 * 24)) + 3_600_000 * 24 // tomorrow_midnight
-        });
-        let start = time_range
+    async fn index(
+        db: Arc<Database>,
+        query_params: IndexQueryParams,
+    ) -> Result<impl Reply, Rejection> {
+        let end = query_params
+            .end
+            .map_or_else(|| Ok(tomorrow_midnight() - 1), |ymd| ymd_midnight(&ymd))
+            .map_err(ClientError::from)?;
+        let start = query_params
             .start
-            .unwrap_or_else(|| end - DEFAULT_SEARCH_INTERVAL);
+            .map_or_else(
+                || Ok(tomorrow_midnight() - DEFAULT_SEARCH_INTERVAL),
+                |ymd| ymd_midnight(&ymd),
+            )
+            .map_err(ClientError::from)?;
+        let keyword = query_params.keyword;
 
         let daily_counts = db
-            .select_daily_count(start, end)
+            .select_daily_count(start, end, keyword.clone())
             .context("daily_count")
             .map_err(ServerError::from)?;
         let (min_time, max_time) = db
@@ -102,11 +120,11 @@ impl Server {
             .context("min_max_time")
             .map_err(ServerError::from)?;
         let title_top100 = db
-            .select_title_top100(start, end)
+            .select_title_top100(start, end, keyword.clone())
             .context("title_top100")
             .map_err(ServerError::from)?;
         let domain_top100 = db
-            .select_domain_top100(start, end)
+            .select_domain_top100(start, end, keyword.clone())
             .context("domain_top100")
             .map_err(ServerError::from)?;
 
@@ -124,9 +142,11 @@ impl Server {
                 max_time => max_time,
                 start => start,
                 end => end,
-                daily_counts => daily_counts.into_iter().map(|dc| (dc.day, dc.count)).collect::<Vec<_>>(),
+                daily_counts => daily_counts,
                 title_top100 => title_top100,
                 domain_top100 => domain_top100,
+                keyword => keyword.unwrap_or_else(|| "".to_string()),
+                version => clap::crate_version!(),
             ))
             .map_err(|e| ServerError::from(Error::from(e)))?;
 
@@ -137,11 +157,12 @@ impl Server {
     fn serve(&self) -> Result<()> {
         let index = warp::path::end()
             .and(Self::with_db(self.db.clone()))
-            .and(warp::query::<TimeRange>())
+            .and(warp::query::<IndexQueryParams>())
             .and_then(Self::index);
 
         let detail = Self::with_db(self.db.clone())
-            .and(warp::path!("details" / i64))
+            .and(warp::path!("details" / String))
+            .and(warp::query::<DetailsQueryParams>())
             .and_then(Self::details);
 
         let static_route = warp::path("static")
@@ -170,6 +191,9 @@ impl Server {
         } else if let Some(ServerError { e }) = err.find() {
             code = StatusCode::INTERNAL_SERVER_ERROR;
             message = e;
+        } else if let Some(ClientError { e }) = err.find() {
+            code = StatusCode::BAD_REQUEST;
+            message = e;
         } else {
             error!("unhandled rejection: {:?}", err);
             code = StatusCode::INTERNAL_SERVER_ERROR;
